# encoding: utf-8
from __future__ import unicode_literals
from collections import namedtuple
from datetime import datetime, time, date
from dateutil.relativedelta import relativedelta

from edu_to_analytics.service.utils import (
    PERIOD_TODAY,
    PERIOD_YESTERDAY,
    PERIOD_DATE,
    PERIOD_PERIOD,
    PERIOD_TYPES,
)

Period = namedtuple('Period', 'type, start, end')


def combined(start, end, period_type):
    """
    Возвращает период времени.

    Особенность в том что дата начала периода должна начинаться
    с 0 часов
    """
    return Period(
        type=period_type,
        start=datetime.combine(start, time(0)),
        end=datetime.combine(end, time(23, 59, 59)))


def period(period_type, end=None, start=date.today()):
    """
    Возвращает периоды опираясь на тип и входные данные.

    Всего может вернуть 4 периода: сегодняшний день, завтра,
    с текущей даты по любую другую и с одной до другой даты.
    """
    from functools import partial

    if period_type not in PERIOD_TYPES:
        raise ValueError(
            'Ошибочный тип периода {}'.format(period_type))

    today = date.today()

    mapping = {
        # при рассчете на сегодня даты значения не имеют
        # оставлено здесь для полноты картины
        PERIOD_TODAY: partial(combined, today, today),
        # для подсчета показателя на вчера считается количество
        # записей в логах на текущую дату чтобы потом из общего
        # количества записей в выборке вычесть данные о текущей активности
        PERIOD_YESTERDAY: partial(
            combined, today - relativedelta(days=1), today),
        # здесь start и end это одна и та же дата
        # как и в случае с подсчетом на вчера берётся разница от текущего
        # значения
        PERIOD_DATE: partial(combined, start, today),
        # период подсчитывается как общий показатель - показания за все дни
        # с конца периода по текущую дату
        PERIOD_PERIOD: partial(combined, start, end),
    }

    # Могли добавить новый тип периода который пока не поддерживается
    if period_type not in mapping:
        raise KeyError('Необходимо обновить маппинг')

    return mapping[period_type](period_type)
