# coding: utf-8
from __future__ import unicode_literals
from __future__ import absolute_import

from lxml.etree import tostring
from spyne.util.xml import get_object_as_xml

from kinder.core.unit.models import Unit, UnitStatus
from kinder.core.dict.models import UnitKind
from kinder.core.emie_unit.models import DopService

from edu_to_analytics.data.utils import UnitIterator, UNIT_FIELDS

from .types import MoItem, DouItem, MoList

from ..utils import AddressBlock


UNIT_ADDRESS_FIELDS = [
    addr_type + scope
    for addr_type in ('address_', 'unitcontingent__jur_address_')
    for scope in ('place', 'street', 'house', 'corps')
]


def get_content():
    region_ids = Unit.objects.filter(
        kind_id=UnitKind.REGION).values_list('id', flat=True)

    list_mo = []

    for region_id in region_ids:
        mos = Unit.objects.filter(kind_id=UnitKind.MO, parent_id=region_id)
        for mo in mos:
            list_dou = []

            units = UnitIterator(region_id, mo.id)
            units.unit_fields = (
                UNIT_FIELDS +
                ['is_filial', 'parent_id', 'status'] +
                UNIT_ADDRESS_FIELDS)
            for unit in units:
                addr = AddressBlock('unit', unit, 'unitcontingent')
                dp_queryset = DopService.objects.filter(
                    unit=unit['id']).values('name')
                list_dou.append(DouItem(
                    id=unit['id'],
                    mainID=unit['parent_id'] if unit['is_filial'] else None,
                    douName=unit['name'],
                    douStatus=UnitStatus.values.get(unit['status'], None),
                    servicesList=[dp['name'] for dp in dp_queryset],
                    addressJur=addr.jur,
                    addressFact=addr.fact,
                ))

            mo_item = MoItem(
                moID=mo.id,
                moName=mo.name,
                douList=list_dou
            )
            list_mo.append(mo_item)

    root = MoList(units=tuple(list_mo))

    result = tostring(
        get_object_as_xml(root), pretty_print=True, encoding='unicode')
    print result
    return result
