# coding: utf-8
from __future__ import unicode_literals
from __future__ import absolute_import

from datetime import datetime, time
from collections import defaultdict

from lxml.etree import tostring
from spyne.util.xml import get_object_as_xml

from django.db.models import Count, When, Case, IntegerField

from kinder.core.unit.models import Unit, UnitStatus
from kinder.core.dict.models import UnitKind
from kinder.core.group.models import Group as GroupModel
from kinder.core.group.enum import GroupStatusEnum

from edu_to_analytics.data.utils import UnitIterator, UNIT_FIELDS

from .types import Root, DouStruct, Group
from ..utils import AddressBlock

UNIT_ADDRESS_FIELDS = [
    'address_place',
    'address_street',
    'address_house',
    'address_corps',
]


def get_groups(decl_date):
    """Получение информации о группах учреждений."""
    date_begin = datetime.combine(decl_date, time.min)
    date_end = datetime.combine(decl_date, time.max)
    groups = GroupModel.objects.exclude(
        status=GroupStatusEnum.ARCHIVE,
    ).annotate(
        pupils=Count('pupil', distinct=True),
        directions=Count('direct', distinct=True),
        # Новые направления созданные на определенную дату
        new_directions=Count(
            Case(
                When(direct__date__range=(date_begin, date_end), then=1),
                output_field=IntegerField()), distinct=True),
    ).values(
        'unit_id',
        'id',
        'cathegory_id',
        'age_cat_id',
        'sub_age_cat_id',
        'work_type_id',
        'health_need_id',
        'max_count_by_area',
        'max_count',
        'pupils',
        'directions',
        'new_directions',
    )

    result = defaultdict(list)
    for g in groups:
        result[g['unit_id']].append(g)
    return result


def get_unit_groups(unit_groups):
    for group in unit_groups:
        yield Group(
            id=group['id'],
            categoryId=group['cathegory_id'],
            ageCategoryId=group['age_cat_id'],
            ageSubCategoryId=group['sub_age_cat_id'],
            workTimeId=group['work_type_id'],
            healthCategoryId=group['health_need_id'],
            capacityByArea=group['max_count_by_area'],
            capacityByFact=group['max_count'],
            pupilAmount=group['pupils'],
            directionsAmount=group['directions'],
            directionsNew=group['new_directions'],
        )


def get_content(ctx, report_code, decl_date):
    """Точка входа метода, возвращает строку xml."""

    dou_units = []
    groups = get_groups(decl_date)
    region_ids = Unit.objects.filter(
        kind_id=UnitKind.REGION).values_list('id', flat=True)
    for region_id in region_ids:
        units = UnitIterator(region_id)
        units.unit_fields = UNIT_FIELDS + [
            'parent_id'] + UNIT_ADDRESS_FIELDS
        for unit in units:
            dou_units.append(DouStruct(
                id=unit['id'],
                mainId=unit['parent_id'],
                addressFact=AddressBlock('unit', unit).fact,
                groups=get_unit_groups(groups[unit['id']]),
            ))

    root = Root(douList=dou_units)
    result = tostring(
        get_object_as_xml(root), pretty_print=True, encoding='unicode')
    print result
    return result